/*
 * Windows Kermit resource file
 *
 * Copyright (c) 1990, 1991 by
 * William S. Hall
 * 3665 Benton Street, #66
 * Santa Clara, CA 95051
 *
 * This file contains the resources needed by the Kermit
 * module.  It must be included along with the necessary
 * header files in the resource file for the terminal
 * program
 *
 */

/* icon */
#ifdef COLUMBIA
Kermit ICON	WKKERM.ICO
#else
Kermit ICON	Kermit.ico
#endif

/* Menu for Kermit.  This item is attached to the main window at init time */
KRMMENU MENU
BEGIN
    POPUP "&Files"
        BEGIN
	    MENUITEM "&Receive", IDM_KRM_RECEIVE
	    MENUITEM "&Send...", IDM_KRM_SEND
	    MENUITEM "&Get...", IDM_KRM_GET
	END
    POPUP "&Remote"
	BEGIN
	    MENUITEM "C&ommand...", IDM_KRM_REMOTECOMMAND
	    MENUITEM SEPARATOR
	    MENUITEM "&Change directory...", IDM_KRM_REMOTECWD
	    MENUITEM "&Directory...", IDM_KRM_REMOTEDIR
	    MENUITEM "&Erase...", IDM_KRM_REMOTEERASE
	    MENUITEM "&Help...", IDM_KRM_REMOTEHELP
	    MENUITEM "&Space...", IDM_KRM_REMOTESPACE
	    MENUITEM "&Type...", IDM_KRM_REMOTETYPE
	    MENUITEM "&Who...", IDM_KRM_REMOTEWHO
	    MENUITEM SEPARATOR
	    MENUITEM "&Finish", IDM_KRM_REMOTEFINISH
	    MENUITEM "&Logout", IDM_KRM_REMOTELOGOUT
	    MENUITEM "&Bye", IDM_KRM_REMOTEBYE
	END
    POPUP "&Local"
	BEGIN
	    MENUITEM "&Directory / Chdir", IDM_KRM_LOCALFILES
	END
    POPUP "&Parameters"
	BEGIN
	    MENUITEM "Set &Protocol...", IDM_KRM_PROTOCOL
	    MENUITEM "Set Pac&kets...", IDM_KRM_PACKETS
	END
    POPUP "&Cancel"
        BEGIN
            MENUITEM "&File", IDM_KRM_FILEABORT, GRAYED
	    MENUITEM "&Batch", IDM_KRM_BATCHABORT, GRAYED
            MENUITEM "&Nicely", IDM_KRM_ERRORABORT, GRAYED
            MENUITEM SEPARATOR
            MENUITEM "&Abruptly", IDM_KRM_CANCEL, GRAYED
	END
END

/* Strings used by Kermit */
STRINGTABLE
BEGIN
  /* Messages */
    IDS_KRM_TRANSACTION_DONE	"Transaction Done"
    IDS_KRM_QUIT		"Abort the current file transfer"
    IDS_KRM_UNKNOWN_PACKET	"Unrecognized Packet Type"
    IDS_KRM_CANCELLED		"Transaction Cancelled"
    IDS_KRM_TOOMANYRETRIES	"Retry Limit Exceeded"
    IDS_KRM_FILE_OPEN_ERROR	"File Open Error"
    IDS_KRM_FILE_READ_ERROR	"File Read Error"
    IDS_KRM_FILE_WRITE_ERROR	"File Write Error"
    IDS_KRM_FILE_CLOSE_ERROR	"File Close Error"
    IDS_KRM_TERMINAL_OPEN_ERROR "Cannot display response from host"
    IDS_KRM_SENDCMD_ERROR	"Command send error"
    IDS_KRM_SENDR_ERROR		"Get file error"
    IDS_KRM_SENDING		"Sending file:"
    IDS_KRM_CANNOTINIT		"Kermit initialization failed"
  /* Menu title */
    IDS_KRM_KERMITMENU		"&Kermit"
  /* WIN.INI strings */
    IDS_KRM_KERMIT		"Kermit"
    IDS_KRM_TIMER		"Timer"
/*    IDS_KRM_VERBOSE		"Verbose"   */
    IDS_KRM_DISCARD		"Discard"
    IDS_KRM_BELL		"Bell"
    IDS_KRM_FILEWARN		"Filewarn"
    IDS_KRM_BLOCKCHECKTYPE	"BlockCheck"
    IDS_KRM_SENDDELAY		"SendDelay"
    IDS_KRM_RETRYLIMIT		"RetryLimit"
    IDS_KRM_SENDMARK		"sMark"
    IDS_KRM_RCVMARK		"rMark"
    IDS_KRM_SENDMAXLEN		"sMaxpktlen"
    IDS_KRM_RCVMAXLEN		"rMaxpktlen"
    IDS_KRM_SENDTIMEOUT		"sTimeOut"
    IDS_KRM_RCVTIMEOUT		"rTimeOut"
    IDS_KRM_SENDPADCOUNT	"sPadCount"
    IDS_KRM_RCVPADCOUNT		"rPadCount"
    IDS_KRM_SENDPADCHAR		"sPadChar"
    IDS_KRM_RCVPADCHAR		"rPadChar"
    IDS_KRM_SENDEOL		"sEol"
    IDS_KRM_RCVEOL		"rEol"
    IDS_KRM_SENDQUOTE		"sQuote"
    IDS_KRM_RCVQUOTE		"rQuote"
    IDS_KRM_REMOTEHELP		"Kermit Remote Help"
    IDS_KRM_HELPTOPIC		"Topic:"
    IDS_KRM_REMOTECOMMAND	"Kermit Remote Command"
    IDS_KRM_CMDTOPIC		"Command:"
    IDS_KRM_REMOTEDIR		"Kermit Remote Directory"
    IDS_KRM_REMOTETYPE		"Kermit Remote Type"
    IDS_KRM_REMOTESPACE		"Kermit Remote Space"
    IDS_KRM_SPACETOPIC		"Area:"
    IDS_KRM_REMOTEWHO		"Kermit Remote Who"
    IDS_KRM_WHOTOPIC		"User:"
    IDS_KRM_REMOTEERASE		"Kermit Remote Erase"
    IDS_KRM_WINDOWOVERFLOW	"Window text has been truncated"
END
