/*
 * Menu and string tables for Win100/Win600
 *
 * Copyright (c) 1990, 1991 by
 * William S. Hall
 * 3665 Benton Street, #66
 * Santa Clara, CA 95051
 */

STRINGTABLE
BEGIN
#ifdef WIN100
    IDS_APPNAME		"Win100"	/* application name */
    IDS_ICON		"w100"		/* icon string */
    IDS_TITLE		"Win100 Terminal Emulator on "	/* title bar */
    IDS_LOGFILENAME	"WIN100.LOG"
#else
    IDS_APPNAME		"Win600"	/* application name */
    IDS_ICON		"W600"		/* icon string */
    IDS_TITLE		"Win600 Terminal Emulator on "	/* title bar */
    IDS_LOGFILENAME	"WIN600.LOG"
#endif
    IDS_ADDDEFAULTS	"Add Terminal Defaults"
    IDS_COM1		"COM1"
    IDS_COM2		"COM2"
    IDS_COM3		"COM3"
    IDS_COM4		"COM4"
    IDS_FIRSTPORT	"Port1"
    IDS_SECONDPORT	"Port2"
    IDS_THIRDPORT	"Port3"
    IDS_FOURTHPORT	"Port4"
    IDS_RXQUESIZE	"RXQLen"
    IDS_TXQUESIZE	"TXQLen"
    IDS_NOCOM		"No communications ports available"
    IDS_CANNOTCLOSECOM	"Cannot close communications port"
    IDS_ROWS		"Rows"
    IDS_COLS		"Columns"
    IDS_NOCOMSET	"Cannot set comm port"
    IDS_BAUD		"Baudrate"
    IDS_PARITY		"Parity"
    IDS_BYTESIZE	"Bytesize"
    IDS_STOPBITS	"Stopbits"
    IDS_PARCHECK	"ParityChk"
    IDS_XONXOFF		"XonXoff"
    IDS_XOFFLIM		"XoffLim"
    IDS_LOCALECHO	"LocalEcho"
    IDS_AUTOLFCR	"AutoLFCR"
    IDS_MARGINBELL	"MarginBell"
    IDS_CURSOR		"Cursor"
    IDS_CURSORTYPE	"BlockCursor"
    IDS_WARNINGBELL	"WarningBell"

    IDS_LINE		"&Line"
    IDS_LOCAL		"&Local"
    IDS_WRAP		"LineWrap"
    IDS_TEXTCOLOR	"TextColor"
    IDS_BGCOLOR		"ScreenColor"
    IDS_TERMTYPE	"TermType"
    IDS_REVERSEVIDEO	"ReverseVideo"
    IDS_SMOOTHSCROLL	"SmoothScroll"
    IDS_SCROLLUNITS	"ScrollUnits"
    IDS_LOCALMODE	"LocalMode"
    IDS_TRYANOTHER	"  Try another port?"
    IDS_LOGSESSION	"&Log session..."
    IDS_CLOSELOG	"Close &log file"
    IDS_FILEEXISTS	"Do you wish to append to this file?"
    IDS_OPENFILEERROR	"Error in opening log file."
    IDS_ALTKEYPAD	"AltKeypad"
    IDS_CURSORKEYS	"CursorKeys"
    IDS_BREAKKEYENABLED "BreakEnabled"
    IDS_DELETEKEY	"BkspIsDelete"
    IDS_AUTOANSWER	"AutoAnswerback"
    IDS_ANSWERBACKKEY	"ABMsg"
    IDS_FONTFACE	"Fontface"
    IDS_DEFAULTFONT	"System"
    IDS_NORMALWIDTH	"NFontWidth"
    IDS_NORMALHEIGHT	"NFontHeight"
    IDS_SMALLWIDTH	"SFontWidth"
    IDS_SMALLHEIGHT	"SFontHeight"   
    IDS_CONTROLREP	"ControlRep"
    IDS_WINHELPNAME	"WINHELP.HLP"
    IDS_EMULATIONWIDTH  "EWidth"
END

/* Strings in dialog boxes */
STRINGTABLE
BEGIN
    IDS_JUMP		"Jump"
    IDS_SMOOTH1		"Smooth 1"
    IDS_SMOOTH2		"Smooth 2"
    IDS_SMOOTH4		"Smooth 4"
    IDS_SMOOTH8		"Smooth 8"
/*
    IDS_VT1007BIT	"VT100 7 bit"
    IDS_VT1008BIT	"VT100 8 bit"
    IDS_VT527BIT	"VT52 7 bit"
    IDS_VT528BIT	"VT52 8 bit"
*/
    IDS_BLACK		"Black"
    IDS_DARKRED		"Dark red"
    IDS_DARKGREEN	"Dark green"
    IDS_DARKYELLOW	"Dark yellow"
    IDS_DARKBLUE	"Dark blue"
    IDS_DARKMAGENTA	"Dark magenta"
    IDS_DARKCYAN	"Dark cyan"
    IDS_DARKGRAY	"Dark gray"
    IDS_LIGHTGRAY	"Light gray"
    IDS_LIGHTRED	"Light red"
    IDS_LIGHTGREEN	"Light green"
    IDS_LIGHTYELLOW	"Light yellow"
    IDS_LIGHTBLUE	"Light blue"
    IDS_LIGHTMAGENTA	"Light magenta"
    IDS_LIGHTCYAN	"Light cyan"
    IDS_WHITE		"White"
END

STRINGTABLE
{
    IE_BADID		"Invalid or unsupported communications port identifier."
    IE_BAUDRATE		"Unsupported communications baud rate."
    IE_BYTESIZE		"Invalid communications byte size."
    IE_DEFAULT		"Error in communications default parameters."
    IE_HARDWARE		"Communications hardware not present."
    IE_MEMORY		"Unable to allocate communications queues."
    IE_NOPEN		"Communications port failed to open."
    IE_OPEN		"Communications port in use by another application."
}

#ifdef WIN100
Win100 MENU
#else
Win600 MENU
#endif
{
    POPUP "&File"
    {
	MENUITEM "&Log session...", IDM_GETFILE
    }
    POPUP "&Edit"
    {
	MENUITEM "Program &function keys...", IDM_EDITFKEYS
	MENUITEM "&Copy line range...", IDM_COPYLINES
	MENUITEM "&Paste", IDM_PASTE, GRAYED
	POPUP "&Buffer"
	{
	    MENUITEM "&Line back\tCtrl-Up", IDM_C_UP
	    MENUITEM "L&ine forward\tCtrl-Dn", IDM_C_DOWN
	    MENUITEM "&Page back\tCtrl-PgUp", IDM_C_PRIOR
	    MENUITEM "P&age forward\tCtrl-PgDn", IDM_C_NEXT
	    MENUITEM "&Top\tCtrl-Home", IDM_C_HOME
	    MENUITEM "&Bottom\tCtrl-End", IDM_C_END
	}
	POPUP "&Window"
	{
	    MENUITEM "&Up\tShift-Up", IDM_S_UP
	    MENUITEM "&Down\tShift-Down", IDM_S_DOWN
	    MENUITEM "&Right\tShift-Right", IDM_S_RIGHT
	    MENUITEM "&Left\tShift-Left", IDM_S_LEFT
	    MENUITEM "&Home\tShift-Home", IDM_S_HOME
	}
    }
    POPUP "&Setup"
    {
	MENUITEM "Mini&Setup...", IDM_MINI
	MENUITEM "&Modem...", IDM_MODEM
	MENUITEM "&Display...", IDM_DISPLAY
	MENUITEM "&General...", IDM_GENERAL
	MENUITEM "&Keyboard...", IDM_KEYBOARD
	MENUITEM "&Fonts...", IDM_FONTS
	MENUITEM SEPARATOR
	MENUITEM "&Recall Saved Setup", IDM_RECALLSAVED
	MENUITEM "&Clear Comm Port", IDM_CLEARCOM
    }
    POPUP "&Help"
    {
/*
	MENUITEM "&Index", IDM_HELP_INDEX, GRAYED
	MENUITEM "&Keyboard", IDM_HELP_KEY, GRAYED
	MENUITEM "&Commands", IDM_HELP_COM, GRAYED
	MENUITEM "Procedures", IDM_HELP_PROC, GRAYED
	MENUITEM "Glossary", IDM_HELP_GLOS, GRAYED
*/
	MENUITEM "&Using Help", IDM_HELP_HELP
	MENUITEM SEPARATOR
#ifdef WIN100
	MENUITEM "&About Win100...", IDM_ABOUT
#else
	MENUITEM "&About Win600...", IDM_ABOUT
#endif
    }
}
#ifdef WIN100
Win100 ACCELERATORS
#else
Win600 ACCELERATORS
#endif
{

    VK_UP,	IDM_UP, VIRTKEY
    VK_DOWN,	IDM_DOWN, VIRTKEY
    VK_RIGHT,	IDM_RIGHT, VIRTKEY
    VK_LEFT,	IDM_LEFT, VIRTKEY
    VK_HOME,	IDM_HOME, VIRTKEY
    VK_END,	IDM_END, VIRTKEY
    VK_PRIOR,	IDM_PRIOR, VIRTKEY
    VK_NEXT,	IDM_NEXT, VIRTKEY
    VK_CLEAR,	IDM_CLEAR, VIRTKEY
    VK_INSERT,  IDM_INSERT, VIRTKEY
    VK_DELETE,  IDM_DELETE, VIRTKEY

    VK_UP,	IDM_S_UP, VIRTKEY,SHIFT
    VK_DOWN,	IDM_S_DOWN, VIRTKEY,SHIFT
    VK_RIGHT,	IDM_S_RIGHT, VIRTKEY,SHIFT
    VK_LEFT,	IDM_S_LEFT, VIRTKEY,SHIFT
    VK_HOME,	IDM_S_HOME, VIRTKEY,SHIFT
    VK_END,	IDM_S_END, VIRTKEY,SHIFT
    VK_PRIOR,	IDM_S_PRIOR, VIRTKEY,SHIFT
    VK_NEXT,	IDM_S_NEXT, VIRTKEY,SHIFT
    VK_CLEAR,	IDM_S_CLEAR, VIRTKEY,SHIFT
    VK_INSERT,  IDM_S_INSERT, VIRTKEY,SHIFT
    VK_DELETE,  IDM_S_DELETE, VIRTKEY,SHIFT

    VK_UP,	IDM_C_UP, VIRTKEY,CONTROL
    VK_DOWN,	IDM_C_DOWN, VIRTKEY,CONTROL
    VK_RIGHT,	IDM_C_RIGHT, VIRTKEY,CONTROL
    VK_LEFT,	IDM_C_LEFT, VIRTKEY,CONTROL
    VK_HOME,	IDM_C_HOME, VIRTKEY,CONTROL
    VK_END,	IDM_C_END, VIRTKEY,CONTROL
    VK_PRIOR,	IDM_C_PRIOR, VIRTKEY,CONTROL
    VK_NEXT,	IDM_C_NEXT, VIRTKEY,CONTROL
    VK_CLEAR,	IDM_C_CLEAR, VIRTKEY,CONTROL
    VK_INSERT,  IDM_C_INSERT, VIRTKEY,CONTROL
    VK_DELETE,  IDM_C_DELETE, VIRTKEY,CONTROL

    VK_UP,	IDM_CS_UP, VIRTKEY,SHIFT,CONTROL
    VK_DOWN,	IDM_CS_DOWN, VIRTKEY,SHIFT,CONTROL
    VK_RIGHT,	IDM_CS_RIGHT, VIRTKEY,SHIFT,CONTROL
    VK_LEFT,	IDM_CS_LEFT, VIRTKEY,SHIFT,CONTROL
    VK_HOME,	IDM_CS_HOME, VIRTKEY,SHIFT,CONTROL
    VK_END,	IDM_CS_END, VIRTKEY,SHIFT,CONTROL
    VK_PRIOR,	IDM_CS_PRIOR, VIRTKEY,SHIFT,CONTROL
    VK_NEXT,	IDM_CS_NEXT, VIRTKEY,SHIFT,CONTROL
    VK_CLEAR,	IDM_CS_CLEAR, VIRTKEY,SHIFT,CONTROL
    VK_INSERT,  IDM_CS_INSERT, VIRTKEY,SHIFT,CONTROL
    VK_DELETE,  IDM_CS_DELETE, VIRTKEY,SHIFT,CONTROL

    VK_F1,	IDM_F1, VIRTKEY
    VK_F2,	IDM_F2, VIRTKEY
    VK_F3,	IDM_F3, VIRTKEY
    VK_F4,	IDM_F4, VIRTKEY
    VK_F5,	IDM_F5, VIRTKEY
    VK_F6,	IDM_F6, VIRTKEY
    VK_F7,	IDM_F7, VIRTKEY
    VK_F8,	IDM_F8, VIRTKEY
    VK_F9,	IDM_F9, VIRTKEY
    VK_F10,	IDM_F10, VIRTKEY
    VK_F11,	IDM_F11, VIRTKEY
    VK_F12,	IDM_F12, VIRTKEY

    VK_F1,	IDM_S_F1, VIRTKEY,SHIFT
    VK_F2,	IDM_S_F2, VIRTKEY,SHIFT
    VK_F3,	IDM_S_F3, VIRTKEY,SHIFT
    VK_F4,	IDM_S_F4, VIRTKEY,SHIFT
    VK_F5,	IDM_S_F5, VIRTKEY,SHIFT
    VK_F6,	IDM_S_F6, VIRTKEY,SHIFT
    VK_F7,	IDM_S_F7, VIRTKEY,SHIFT
    VK_F8,	IDM_S_F8, VIRTKEY,SHIFT
    VK_F9,	IDM_S_F9, VIRTKEY,SHIFT
    VK_F10,	IDM_S_F10, VIRTKEY,SHIFT
    VK_F11,	IDM_S_F11, VIRTKEY,SHIFT
    VK_F12,	IDM_S_F12, VIRTKEY,SHIFT

    VK_F1,	IDM_C_F1, VIRTKEY,CONTROL
    VK_F2,	IDM_C_F2, VIRTKEY,CONTROL
    VK_F3,	IDM_C_F3, VIRTKEY,CONTROL
    VK_F4,	IDM_C_F4, VIRTKEY,CONTROL
    VK_F5,	IDM_C_F5, VIRTKEY,CONTROL
    VK_F6,	IDM_C_F6, VIRTKEY,CONTROL
    VK_F7,	IDM_C_F7, VIRTKEY,CONTROL
    VK_F8,	IDM_C_F8, VIRTKEY,CONTROL
    VK_F9,	IDM_C_F9, VIRTKEY,CONTROL
    VK_F10,	IDM_C_F10, VIRTKEY,CONTROL
    VK_F11,	IDM_C_F11, VIRTKEY,CONTROL
    VK_F12,	IDM_C_F12, VIRTKEY,CONTROL

    VK_F1,	IDM_CS_F1, VIRTKEY,CONTROL,SHIFT
    VK_F2,	IDM_CS_F2, VIRTKEY,CONTROL,SHIFT
    VK_F3,	IDM_CS_F3, VIRTKEY,CONTROL,SHIFT
    VK_F4,	IDM_CS_F4, VIRTKEY,CONTROL,SHIFT
    VK_F5,	IDM_CS_F5, VIRTKEY,CONTROL,SHIFT
    VK_F6,	IDM_CS_F6, VIRTKEY,CONTROL,SHIFT
    VK_F7,	IDM_CS_F7, VIRTKEY,CONTROL,SHIFT
    VK_F8,	IDM_CS_F8, VIRTKEY,CONTROL,SHIFT
    VK_F9,	IDM_CS_F9, VIRTKEY,CONTROL,SHIFT
    VK_F10,	IDM_CS_F10, VIRTKEY,CONTROL,SHIFT
    VK_F11,	IDM_CS_F11, VIRTKEY,CONTROL,SHIFT
    VK_F12,	IDM_CS_F12, VIRTKEY,CONTROL,SHIFT

    VK_NUMPAD0,	IDM_NUM0, VIRTKEY
    VK_NUMPAD1,	IDM_NUM1, VIRTKEY
    VK_NUMPAD2,	IDM_NUM2, VIRTKEY
    VK_NUMPAD3,	IDM_NUM3, VIRTKEY
    VK_NUMPAD4,	IDM_NUM4, VIRTKEY
    VK_NUMPAD5,	IDM_NUM5, VIRTKEY
    VK_NUMPAD6,	IDM_NUM6, VIRTKEY
    VK_NUMPAD7,	IDM_NUM7, VIRTKEY
    VK_NUMPAD8,	IDM_NUM8, VIRTKEY
    VK_NUMPAD9,	IDM_NUM9, VIRTKEY
    VK_ADD,	IDM_ADD, VIRTKEY
    VK_SUBTRACT, IDM_SUBTRACT, VIRTKEY
    VK_DECIMAL, IDM_DECIMAL, VIRTKEY
    VK_MULTIPLY, IDM_MULTIPLY, VIRTKEY

    VK_NUMPAD0,	IDM_S_NUM0, VIRTKEY,SHIFT
    VK_NUMPAD1,	IDM_S_NUM1, VIRTKEY,SHIFT
    VK_NUMPAD2,	IDM_S_NUM2, VIRTKEY,SHIFT
    VK_NUMPAD3,	IDM_S_NUM3, VIRTKEY,SHIFT
    VK_NUMPAD4,	IDM_S_NUM4, VIRTKEY,SHIFT
    VK_NUMPAD5,	IDM_S_NUM5, VIRTKEY,SHIFT
    VK_NUMPAD6,	IDM_S_NUM6, VIRTKEY,SHIFT
    VK_NUMPAD7,	IDM_S_NUM7, VIRTKEY,SHIFT
    VK_NUMPAD8,	IDM_S_NUM8, VIRTKEY,SHIFT
    VK_NUMPAD9,	IDM_S_NUM9, VIRTKEY,SHIFT
    VK_ADD,	IDM_S_ADD, VIRTKEY,SHIFT
    VK_SUBTRACT, IDM_S_SUBTRACT, VIRTKEY,SHIFT
    VK_DECIMAL, IDM_S_DECIMAL, VIRTKEY,SHIFT
    VK_MULTIPLY, IDM_S_MULTIPLY, VIRTKEY,SHIFT

    VK_NUMPAD0,	IDM_C_NUM0, VIRTKEY,CONTROL
    VK_NUMPAD1,	IDM_C_NUM1, VIRTKEY,CONTROL
    VK_NUMPAD2,	IDM_C_NUM2, VIRTKEY,CONTROL
    VK_NUMPAD3,	IDM_C_NUM3, VIRTKEY,CONTROL
    VK_NUMPAD4,	IDM_C_NUM4, VIRTKEY,CONTROL
    VK_NUMPAD5,	IDM_C_NUM5, VIRTKEY,CONTROL
    VK_NUMPAD6,	IDM_C_NUM6, VIRTKEY,CONTROL
    VK_NUMPAD7,	IDM_C_NUM7, VIRTKEY,CONTROL
    VK_NUMPAD8,	IDM_C_NUM8, VIRTKEY,CONTROL
    VK_NUMPAD9,	IDM_C_NUM9, VIRTKEY,CONTROL
    VK_ADD,	IDM_C_ADD, VIRTKEY,CONTROL
    VK_SUBTRACT, IDM_C_SUBTRACT, VIRTKEY,CONTROL
    VK_DECIMAL, IDM_C_DECIMAL, VIRTKEY,CONTROL
    VK_MULTIPLY, IDM_C_MULTIPLY, VIRTKEY,CONTROL

    VK_NUMPAD0,	IDM_CS_NUM0, VIRTKEY,CONTROL,SHIFT
    VK_NUMPAD1,	IDM_CS_NUM1, VIRTKEY,CONTROL,SHIFT
    VK_NUMPAD2,	IDM_CS_NUM2, VIRTKEY,CONTROL,SHIFT
    VK_NUMPAD3,	IDM_CS_NUM3, VIRTKEY,CONTROL,SHIFT
    VK_NUMPAD4,	IDM_CS_NUM4, VIRTKEY,CONTROL,SHIFT
    VK_NUMPAD5,	IDM_CS_NUM5, VIRTKEY,CONTROL,SHIFT
    VK_NUMPAD6,	IDM_CS_NUM6, VIRTKEY,CONTROL,SHIFT
    VK_NUMPAD7,	IDM_CS_NUM7, VIRTKEY,CONTROL,SHIFT
    VK_NUMPAD8,	IDM_CS_NUM8, VIRTKEY,CONTROL,SHIFT
    VK_NUMPAD9,	IDM_CS_NUM9, VIRTKEY,CONTROL,SHIFT
    VK_ADD,	IDM_CS_ADD, VIRTKEY,CONTROL,SHIFT
    VK_SUBTRACT, IDM_CS_SUBTRACT, VIRTKEY,CONTROL,SHIFT
    VK_DECIMAL, IDM_CS_DECIMAL, VIRTKEY,CONTROL,SHIFT
    VK_MULTIPLY, IDM_CS_MULTIPLY, VIRTKEY,CONTROL,SHIFT

}

