#define IDOK                1
#define IDCANCEL            2
#define IDD_SAVE            3
#define IDD_PROGRAM         10
#define IDD_VERSION         11
#define IDD_LINE            100
#define IDD_LOCAL           101
#define IDD_NORMAL          102
#define IDD_REVERSE         103
#define IDD_80COL           104
#define IDD_132COL          105
#define IDD_SCROLL          106
#define IDD_AUTOWRAP        200
#define IDD_AUTOLFCR        201
#define IDD_CTRLREP         202
#define IDD_MARGINBELL      203
#define IDD_CURSOROFF       204
#define IDD_CURSORON        205
#define IDD_CURSORULINE     206
#define IDD_CURSORBLOCK     207
#define IDD_CONTROLREP      208
#define IDD_110BAUD         300
#define IDD_134BAUD         301
#define IDD_300BAUD         302
#define IDD_600BAUD         303
#define IDD_1200BAUD        304
#define IDD_2400BAUD        305
#define IDD_4800BAUD        306
#define IDD_9600BAUD        307
#define IDD_19200BAUD       308
#define IDD_8BITNONE        310
#define IDD_8BITODD         311
#define IDD_8BITEVEN        312
#define IDD_7BITNONE        313
#define IDD_7BITODD         314
#define IDD_7BITEVEN        315
#define IDD_7BITMARK        316
#define IDD_7BITSPACE       317
#define IDD_PARCHECK        318
#define IDD_ONESTOPBIT      319
#define IDD_TWOSTOPBITS     320
#define IDD_XONXOFF         321
#define IDD_XOFF64          322
#define IDD_XOFF128         323
#define IDD_XOFF256         324
#define IDD_XOFF512         325
#define IDD_XOFF768         326
#define IDD_PORT            327
#define IDD_DEFPORT         328
#define IDD_LOCALECHO       331
#define IDD_VT52            400
#define IDD_VT100           401
#define IDD_WARNINGBELL     402
#define IDD_SCREENCOLOR     403
#define IDD_TEXTCOLOR       404
#define IDD_7BITS           419
#define IDD_8BITS           420
#define IDD_STARTLINE       500
#define IDD_ENDLINE         501
#define IDD_STARTSCROLL     502
#define IDD_ENDSCROLL       503
#define IDD_F1              600
#define IDD_F2              601
#define IDD_F3              602
#define IDD_F4              603
#define IDD_F5              604
#define IDD_F6              605
#define IDD_F7              606
#define IDD_F8              607
#define IDD_F9              608
#define IDD_F10             609
#define IDD_F11             610
#define IDD_F12             611
#define IDD_UNSHIFTED       612
#define IDD_SHIFT           613
#define IDD_CONTROL         614
#define IDD_CTRLSHIFT       615
#define IDD_STRINGEDIT      616
#define IDD_GETFILE         700
#define IDD_LOGFILEPATH     701
#define IDD_KEYPADNUMERIC   800
#define IDD_KEYPADAPPL      801
#define IDD_CURKEYSNORMAL   802
#define IDD_CURKEYSAPPL     803
#define IDD_BKSPNORMAL      804
#define IDD_BKSPDELETE      805
#define IDD_AUTOANSWER      806
#define IDD_ANSWERBACK      807
#define IDD_BREAKKEY        808
#define IDD_FONTFACE        900
#define IDD_NORMALFONT      901
#define IDD_SMALLFONT       902

