; Kermit definitions file
;
; Copyright (c) 1990, 1991 by
; William S. Hall
; 3665 Benton Street #66
; Santa Clara, CA 95051
;
; The contents of this file must be
; appended to the main module's definition file
;
SEGMENTS
    KRM_WNKERM LOADONCALL MOVEABLE DISCARDABLE
    KRM_WNKINT LOADONCALL MOVEABLE DISCARDABLE
    KRM_WNKMSC LOADONCALL MOVEABLE DISCARDABLE

EXPORTS
    krmDoTimeout	@1000
    krmXferDlgBox	@1001
    krmCancelBoxProc	@1002
    krmSendFileProc	@1003
    krmProtocol		@1004
    krmPackets		@1005
    krmCWD		@1006
    krmXferRemote	@1007
    krmRemote1ParamCmd  @1008
    krmLocalFiles	@1009
