/*
 * Base resource file for WIN100
 * 
 * Copyright (c) 1990, 1991 by
 * William S. Hall
 * 3665 Benton Street  #66
 * Santa Clara, CA 95051
 *
 */

#define WIN100
#define WIN3

#include <windows.h>
#ifdef COLUMBIA
#include "wkt100.h"
#include "wktdlg.h"
#include "wkkerm.h"
#include "wkkdlg.h"
#include "wktmnu.rc"
#include "wktdlg.dlg"
#include "wkkmnu.rc"
#include "wkkerm.dlg"
#else
#include "..\win600.h"
#include "..\w600dg.h"
#include "..\..\wnkerm\\wnkerm.h"
#include "..\..\wnkerm\\wnkdlg.h"

#include "..\win600mn.rc"
#include "..\w600dg.dlg"

rcinclude ..\..\wnkerm\wnkmnu.rc
rcinclude ..\..\wnkerm\wnkerm.dlg
#endif


