; Definitions file for the Windows Win100 emulator
;
; Copyright (c) 1990, 1991 by
; William S. Hall
; 3665 Benton Street  #66
; Santa Clara, CA 95051

; Program name
NAME

; Description string
DESCRIPTION 'Win100 terminal emulator'

; Program to run if run from MS-DOS
STUB 'Winstub.exe'

; Heap and stack sizes
HEAPSIZE	1024
STACKSIZE	4096

; Type of executable
EXETYPE		WINDOWS

; Attributes of code and data segments
CODE	MOVEABLE
DATA	MOVEABLE MULTIPLE

; Attributes of particular segments
SEGMENTS
    _Win100RES PRELOAD MOVEABLE
    _Win100NRES LOADONCALL MOVEABLE DISCARDABLE
    _Win100INIT LOADONCALL MOVEABLE DISCARDABLE
    _VT100 LOADONCALL MOVEABLE DISCARDABLE
    _VT52 LOADONCALL MOVEABLE DISCARDABLE
    _SMARTTERM PRELOAD MOVEABLE DISCARDABLE

; Functions called by Windows
EXPORTS
    MainWndProc		@1
    MainHasResized	@2
    AboutBoxProc	@3
    ColorChange		@4
    ModemBoxProc	@5
    DisplayBoxProc	@6
    GeneralBoxProc	@7
    MiniBoxProc		@8
    CopyBoxProc		@9
    EditFunctionKeys	@10
    LogFileBoxProc	@11
    KeyboardBoxProc	@12
    FontsBoxProc	@13
    GetFixedFontInfo	@14

; Functions exported by smart terminal code
    SmartTermWndProc	@100
    GetFontInfo		@101
