#define IDD_KRM_OPERATION         100
#define IDD_KRM_FILENAME          101
#define IDD_KRM_SAVENAME          102
#define IDD_KRM_BYTESMOVED        103
#define IDD_KRM_PACKETS           104
#define IDD_KRM_PERCENTAGE        106
#define IDD_KRM_RETRIES           107
#define IDD_KRM_CANCEL            108
#define IDD_KRM_CLOSE             109
#define IDD_KRM_SAVE              110
#define IDD_KRM_CANCELFILE        200
#define IDD_KRM_CANCELBATCH       201
#define IDD_KRM_CANCELERROR       202
#define IDD_KRM_CANCELPROTOCOL    203
#define IDD_KRM_PATHDISPLAY       300
#define IDD_KRM_EDITSELECT        301
#define IDD_KRM_LISTSELECT        302
#define IDD_KRM_PATHSELECT        303
#define IDD_KRM_SEND              304
#define IDD_KRM_BELL              400
#define IDD_KRM_TIMER             401
#define IDD_KRM_DISCARD           402
#define IDD_KRM_FILEWARNING       403
#define IDD_KRM_SENDDELAY         405
#define IDD_KRM_RETRYLIMIT        406
#define IDD_KRM_VERBOSE           407
#define IDD_KRM_1BYTECHK          408
#define IDD_KRM_2BYTECHK          409
#define IDD_KRM_3BYTECHK          410
#define IDD_KRM_SENDMARK          500
#define IDD_KRM_RECEIVEMARK       501
#define IDD_KRM_SENDMAXSIZE       502
#define IDD_KRM_RECEIVEMAXSIZE    503
#define IDD_KRM_SENDTIMEOUT       504
#define IDD_KRM_RECEIVETIMEOUT    505
#define IDD_KRM_SENDPADCOUNT      506
#define IDD_KRM_RECEIVEPADCOUNT   507
#define IDD_KRM_SENDPADCHAR       508
#define IDD_KRM_RECEIVEPADCHAR    509
#define IDD_KRM_SENDEOL           510
#define IDD_KRM_RECEIVEEOL        511
#define IDD_KRM_SENDQUOTE         512
#define IDD_KRM_RECEIVEQUOTE      513
#define IDD_KRM_PASSWORD          600
#define IDD_KRM_EDITTITLE         601
#define IDD_KRM_REMOTEDATA        700
#define IDD_KRM_MESSAGE           701
#define IDD_KRM_LOCALPATH         800
#define IDD_KRM_LOCALFILES        801
#define IDD_KRM_LOCALDIRS         802
#define IDD_KRM_LOCALPROPS        803
#define IDD_KRM_LOCALPATHEDIT     804

